@@ -13,7 +13,6 @@
 ##############################################################################
 """Command-line option parsing
 """
-from __future__ import print_function
 
 import argparse
 import os
@@ -297,6 +296,11 @@ This option is enabled by default if --subunit or --subunit-v2 is in use, to
 avoid corrupting the subunit stream.
 """)
 
+reporting.add_argument(
+    '--xml', action='store', dest='xmlOutput', help="""\
+If given, XML reports will be written to the specified directory.
+""")
+
 
 ######################################################################
 # Analysis
@@ -600,8 +604,7 @@ def get_options(args=None, defaults=None):
     if options.showversion:
         dist = pkg_resources.require('zope.testrunner')[0]
         print('zope.testrunner version %s' % dist.version)
-        options.fail = True
-        return options
+        sys.exit(0)
 
     if options.subunit or options.subunit_v2:
         try:
