@@ -13,13 +13,12 @@
 ##############################################################################
 """Test execution
 """
-from __future__ import print_function
-
 import errno
 import gc
 import io
 import os
 import pprint
+import queue
 import re
 import subprocess
 import sys
@@ -29,8 +28,8 @@ import traceback
 import unittest
 import warnings
 from contextlib import contextmanager
-
-from six import StringIO
+from io import StringIO
+from pathlib import Path
 
 import zope.testrunner
 import zope.testrunner._doctest
@@ -51,6 +50,7 @@ from zope.testrunner import threadsupport
 from zope.testrunner.find import _layer_name_cache
 from zope.testrunner.find import import_name
 from zope.testrunner.find import name_from_layer
+from zope.testrunner.formatter import XMLOutputFormattingWrapper
 from zope.testrunner.layer import EmptyLayer
 from zope.testrunner.layer import EmptySuite
 from zope.testrunner.layer import UnitTests
@@ -62,13 +62,6 @@ from .util import is_jython
 from .util import uses_refcounts
 
 
-try:
-    import Queue  # Python 2
-except ImportError:
-    # Python 3
-    import queue as Queue  # Python 3
-
-
 class UnexpectedSuccess(Exception):
     pass
 
@@ -85,14 +78,14 @@ class SubprocessError(Exception):
         self.stderr = stderr
 
     def __str__(self):
-        return '%s: %s' % (self.reason, self.stderr)
+        return '{}: {}'.format(self.reason, self.stderr)
 
 
 class CanNotTearDown(Exception):
     "Couldn't tear down a test"
 
 
-class Runner(object):
+class Runner:
     """The test runner.
 
     It is the central point of this package and responsible for finding and
@@ -156,8 +149,8 @@ class Runner(object):
             # but only if this is not in the subprocess
             yield (name_from_layer(EmptyLayer), EmptyLayer, EmptySuite())
 
-        layer_names = dict([(layer_from_name(layer_name), layer_name)
-                            for layer_name in self.tests_by_layer_name])
+        layer_names = {layer_from_name(layer_name): layer_name
+                       for layer_name in self.tests_by_layer_name}
         for layer in order_by_bases(layer_names):
             layer_name = layer_names[layer]
             yield layer_name, layer, self.tests_by_layer_name[layer_name]
@@ -209,6 +202,9 @@ class Runner(object):
                 for feature in self.features:
                     feature.report()
 
+        if self.options.xmlOutput:
+            self.options.output.writeXMLReports()
+
     def configure(self):
         if self.args is None:
             self.args = sys.argv[:]
@@ -233,6 +229,12 @@ class Runner(object):
         options.resume_layer = resume_layer
         options.resume_number = resume_number
 
+        if options.xmlOutput:
+            folder = Path(options.xmlOutput).resolve()
+            folder.mkdir(parents=True, exist_ok=True)
+            options.output = XMLOutputFormattingWrapper(
+                options.output, folder=folder)
+
         self.options = options
 
         self.features.append(zope.testrunner.selftest.SelfTest(self))
@@ -412,11 +414,8 @@ def run_tests(options, tests, name, failures, errors, skipped, import_errors):
         output.stop_tests()
         failures.extend(result.failures)
         n_failures = len(result.failures)
-        if hasattr(result, 'unexpectedSuccesses'):
-            # Python versions prior to 2.7 do not have the concept of
-            # unexpectedSuccesses.
-            failures.extend(result.unexpectedSuccesses)
-            n_failures += len(result.unexpectedSuccesses)
+        failures.extend(result.unexpectedSuccesses)
+        n_failures += len(result.unexpectedSuccesses)
         skipped.extend(result.skipped)
         errors.extend(result.errors)
         output.summary(n_tests=result.testsRun,
@@ -463,7 +462,7 @@ def run_layer(options, layer_name, layer, tests, setup_layers,
     output = options.output
     gathered = []
     gather_layers(layer, gathered)
-    needed = dict([(ly, 1) for ly in gathered])
+    needed = {ly: 1 for ly in gathered}
     if options.resume_number != 0:
         output.info("Running %s tests:" % layer_name)
     tear_down_unneeded(options, needed, setup_layers, errors)
@@ -490,7 +489,7 @@ class SetUpLayerFailure(unittest.TestCase):
     subunit_label = 'setUp'
 
     def __init__(self, layer):
-        super(SetUpLayerFailure, self).__init__()
+        super().__init__()
         self.layer = layer
 
     def runTest(self):
@@ -505,7 +504,7 @@ class TearDownLayerFailure(unittest.TestCase):
     subunit_label = 'tearDown'
 
     def __init__(self, layer):
-        super(TearDownLayerFailure, self).__init__()
+        super().__init__()
         self.layer = layer
 
     def runTest(self):
@@ -532,11 +531,6 @@ def spawn_layer_in_subprocess(result, script_parts, options, features,
 
         args.extend(options.original_testrunner_args[1:])
 
-        # this is because of a bug in Python (http://www.python.org/sf/900092)
-        if (options.profile == 'hotshot'
-                and sys.version_info[:3] <= (2, 4, 1)):
-            args.insert(1, '-O')
-
         debugargs = args  # save them before messing up for windows
         if sys.platform.startswith('win'):
             args = args[0] + ' ' + ' '.join([
@@ -575,7 +569,7 @@ def spawn_layer_in_subprocess(result, script_parts, options, features,
                     if not line:
                         break
                     result.write(line)
-            except IOError as e:
+            except OSError as e:
                 if e.errno == errno.EINTR:
                     # If the subprocess dies before we finish reading its
                     # output, a SIGCHLD signal can interrupt the reading.
@@ -664,7 +658,7 @@ def _get_output_buffer(stream):
     return stream
 
 
-class AbstractSubprocessResult(object):
+class AbstractSubprocessResult:
     """A result of a subprocess layer run."""
 
     num_ran = 0
@@ -691,7 +685,7 @@ class ImmediateSubprocessResult(AbstractSubprocessResult):
     """Sends complete output to queue."""
 
     def __init__(self, layer_name, queue):
-        super(ImmediateSubprocessResult, self).__init__(layer_name, queue)
+        super().__init__(layer_name, queue)
         self.stream = _get_output_buffer(sys.stdout)
 
     def write(self, out):
@@ -730,7 +724,7 @@ def resume_tests(script_parts, options, features, layers, failures, errors,
     elif (options.verbose > 1 and
             not options.subunit and not options.subunit_v2):
         result_factory = KeepaliveSubprocessResult
-        stdout_queue = Queue.Queue()
+        stdout_queue = queue.Queue()
     else:
         result_factory = DeferredSubprocessResult
     resume_number = int(options.processes > 1)
@@ -767,7 +761,7 @@ def resume_tests(script_parts, options, features, layers, failures, errors,
             previous_output = output
             try:
                 layer_name, output = stdout_queue.get(False)
-            except Queue.Empty:
+            except queue.Empty:
                 break
             if layer_name != last_layer_intermediate_output:
                 # Clarify what layer is reporting activity.
@@ -908,20 +902,17 @@ class TestResult(unittest.TestResult):
 
     def _makeBufferedStdStream(self):
         """Make a buffered stream to replace a standard stream."""
-        if sys.version_info[0] >= 3:
-            # The returned stream needs to have a 'buffer' attribute, since
-            # some tests may expect that to exist on standard streams, and a
-            # 'getvalue' method for the convenience of _restoreStdStreams.
-            # This requires some care.
-            class BufferedStandardStream(io.TextIOWrapper):
-                def getvalue(self):
-                    return self.buffer.getvalue().decode(
-                        encoding=self.encoding, errors=self.errors)
+        # The returned stream needs to have a 'buffer' attribute, since
+        # some tests may expect that to exist on standard streams, and a
+        # 'getvalue' method for the convenience of _restoreStdStreams.
+        # This requires some care.
+        class BufferedStandardStream(io.TextIOWrapper):
+            def getvalue(self):
+                return self.buffer.getvalue().decode(
+                    encoding=self.encoding, errors=self.errors)
 
-            return BufferedStandardStream(
-                io.BytesIO(), newline='\n', write_through=True)
-        else:
-            return StringIO()
+        return BufferedStandardStream(
+            io.BytesIO(), newline='\n', write_through=True)
 
     def _setUpStdStreams(self):
         """Set up buffered standard streams, if requested."""
@@ -969,7 +960,17 @@ class TestResult(unittest.TestResult):
         self.options.output.test_success(test, t)
 
     def addSkip(self, test, reason):
-        self._restoreStdStreams()
+        if not hasattr(self, "_test_state"):
+            # ``startTest`` was not called -- set up extected state
+            self._test_state = test.__dict__
+            count = test.countTestCases()
+            self.testsRun += count
+            self.options.output.start_test(test, self.testsRun, self.count)
+            self._threads = threadsupport.enumerate()
+            if not hasattr(self, "_start_time"):
+                self._start_time = time.time()
+        else:
+            self._restoreStdStreams()
         unittest.TestResult.addSkip(self, test, reason)
         self.options.output.test_skipped(test, reason)
 
@@ -1054,8 +1055,6 @@ class TestResult(unittest.TestResult):
                 cycles = [[repr_lines(o) for o in c] for c in g.sccs()]
                 del gc.garbage[:]
                 g = obj = None  # avoid to hold cyclic garbage
-                # Python 2: avoid to hold cyclic garbage
-                o = c = None  # noqa: F841
             gc.set_debug(gc_opts)
         gccount = gc.collect() \
             if uses_refcounts and self.options.gc_after_test else 0
@@ -1114,7 +1113,7 @@ def layer_sort_key(layer):
     Based on the reverse MRO ordering in order to put layers with shared base
     layers next to each other.
     """
-    seen = set([])
+    seen = set()
     key = []
 
     # Note: we cannot reuse gather_layers() here because it uses a
@@ -1193,7 +1192,7 @@ def repr_lines(obj, max_width=75, max_lines=5):
         oi = pprint.pformat(obj)
     except Exception:
         # unprintable
-        oi = "%s instance at 0x%x" % (obj.__class__, id(obj))
+        oi = "{} instance at 0x{:x}".format(obj.__class__, id(obj))
     # limit
     cmps = oi.split("\n", max_lines)
     if len(cmps) > max_lines:
